@@ -15,7 +15,7 @@ from wisdem.rotorse.rotor_cost import blade_cost_model
 
 from scipy.interpolate import PchipInterpolator
 
-NINPUT = 5
+NINPUT = 8
 
 
 
@@ -24,12 +24,13 @@ class BladeGeometry(ExplicitComponent):
         self.options.declare('RefBlade')
 
         # Blade Cost Model Options
-        self.options.declare('verbosity',        default=False)
-        self.options.declare('tex_table',        default=False)
-        self.options.declare('generate_plots',   default=False)
-        self.options.declare('show_plots',       default=False)
-        self.options.declare('show_warnings',    default=False)
-        self.options.declare('discrete',         default=False)
+        self.options.declare('verbosity',           default=False)
+        self.options.declare('tex_table',           default=False)
+        self.options.declare('generate_plots',      default=False)
+        self.options.declare('show_plots',          default=False)
+        self.options.declare('show_warnings',       default=False)
+        self.options.declare('discrete',            default=False)
+        self.options.declare('user_update_routine', default=None)
     
     def setup(self):
         self.refBlade = RefBlade = self.options['RefBlade']
@@ -47,6 +48,7 @@ class BladeGeometry(ExplicitComponent):
         self.add_input('presweep_in',   val=np.zeros(NINPUT), units='m', desc='precurve at control points')  # defined at same locations at chord, starting at 2nd control point (root must be zero precurve)
         self.add_input('sparT_in',      val=np.zeros(NINPUT), units='m', desc='thickness values of spar cap that linearly vary from non-cylinder position to tip')
         self.add_input('teT_in',        val=np.zeros(NINPUT), units='m', desc='thickness values of trailing edge panels that linearly vary from non-cylinder position to tip')
+        # self.add_input('leT_in',        val=np.zeros(NINPUT), units='m', desc='thickness values of leading edge panels that linearly vary from non-cylinder position to tip')
         self.add_input('airfoil_position', val=np.zeros(NAF), desc='spanwise position of airfoils')
 
         # parameters
@@ -142,6 +144,7 @@ class BladeGeometry(ExplicitComponent):
         blade['ctrl_pts']['presweep_in']  = inputs['presweep_in']
         blade['ctrl_pts']['sparT_in']     = inputs['sparT_in']
         blade['ctrl_pts']['teT_in']       = inputs['teT_in']
+        # blade['ctrl_pts']['leT_in']       = inputs['leT_in']
         blade['ctrl_pts']['r_max_chord']  = inputs['r_max_chord']
 
         #check that airfoil positions are increasing        
@@ -160,13 +163,16 @@ class BladeGeometry(ExplicitComponent):
         
         # Update
         refBlade = ReferenceBlade()
-        refBlade.verbose        = False
-        refBlade.NINPUT         = len(outputs['r_in'])
-        refBlade.NPTS           = len(blade['pf']['s'])
-        refBlade.analysis_level = blade['analysis_level']
+        refBlade.verbose             = False
+        refBlade.NINPUT              = len(outputs['r_in'])
+        refBlade.NPTS                = len(blade['pf']['s'])
+        refBlade.analysis_level      = blade['analysis_level']
+        refBlade.user_update_routine = self.options['user_update_routine']
         if blade['analysis_level'] < 3:
-            refBlade.spar_var   = blade['precomp']['spar_var']
-            refBlade.te_var     = blade['precomp']['te_var']
+            refBlade.spar_var        = blade['precomp']['spar_var']
+            refBlade.te_var          = blade['precomp']['te_var']
+            # if 'le_var' in blade['precomp']:
+            #     refBlade.le_var     = blade['precomp']['le_var']
         
         blade_out = refBlade.update(blade)
         
@@ -336,6 +342,7 @@ class RotorGeometry(Group):
         self.options.declare('show_plots',       default=False)
         self.options.declare('show_warnings',    default=False)
         self.options.declare('discrete',         default=False)
+        self.options.declare('user_update_routine', default=None)
         
     def setup(self):
         RefBlade = self.options['RefBlade']
@@ -343,12 +350,13 @@ class RotorGeometry(Group):
         NINPUT = len(RefBlade['ctrl_pts']['r_in'])
         NAF    = len(RefBlade['outer_shape_bem']['airfoil_position']['grid'])
         
-        verbosity      = self.options['verbosity']
-        tex_table      = self.options['tex_table']     
-        generate_plots = self.options['generate_plots']
-        show_plots     = self.options['show_plots']
-        show_warnings  = self.options['show_warnings']
-        discrete       = self.options['discrete']
+        verbosity           = self.options['verbosity']
+        tex_table           = self.options['tex_table']     
+        generate_plots      = self.options['generate_plots']
+        show_plots          = self.options['show_plots']
+        show_warnings       = self.options['show_warnings']
+        discrete            = self.options['discrete']
+        user_update_routine = self.options['user_update_routine']
         
         # Independent variables that are unique to TowerSE
         if topLevelFlag:
@@ -373,6 +381,7 @@ class RotorGeometry(Group):
             geomIndeps.add_output('airfoil_position', val=np.zeros(NAF))
             geomIndeps.add_output('sparT_in', val=np.zeros(NINPUT), units='m', desc='spar cap thickness parameters')
             geomIndeps.add_output('teT_in', val=np.zeros(NINPUT), units='m', desc='trailing-edge thickness parameters')
+            # geomIndeps.add_output('leT_in', val=np.zeros(NINPUT), units='m', desc='leading-edge thickness parameters')
             self.add_subsystem('geomIndeps', geomIndeps, promotes=['*'])
             
         # --- Rotor Definition ---
@@ -385,7 +394,8 @@ class RotorGeometry(Group):
                                                generate_plots=generate_plots,
                                                show_plots=show_plots,
                                                show_warnings =show_warnings ,
-                                               discrete=discrete), promotes=['*'])
+                                               discrete=discrete,
+                                               user_update_routine=user_update_routine), promotes=['*'])
         self.add_subsystem('geom', CCBladeGeometry(NINPUT = NINPUT), promotes=['precone','precurve_in', 'presweep_in',
                                                                                'precurveTip','presweepTip','R','Rtip'])
 
@@ -400,6 +410,8 @@ def Init_RotorGeometry_wRefBlade(rotor, blade):
     rotor['presweep_in']      = np.array(blade['ctrl_pts']['presweep_in']) #np.array([0.0, 0.0, 0.0])  # (Array, m): precurve at control points.  defined at same locations at chord, starting at 2nd control point (root must be zero precurve)
     rotor['sparT_in']         = np.array(blade['ctrl_pts']['sparT_in']) # np.array([0.0, 0.05, 0.047754, 0.045376, 0.031085, 0.0061398])  # (Array, m): spar cap thickness parameters
     rotor['teT_in']           = np.array(blade['ctrl_pts']['teT_in']) # np.array([0.0, 0.1, 0.09569, 0.06569, 0.02569, 0.00569])  # (Array, m): trailing-edge thickness parameters
+    # if 'le_var' in blade['precomp']['le_var']:
+    #     rotor['leT_in']       = np.array(blade['ctrl_pts']['leT_in']) ## (Array, m): leading-edge thickness parameters
     rotor['airfoil_position'] = np.array(blade['outer_shape_bem']['airfoil_position']['grid'])
     rotor['hubFraction']      = blade['config']['hubD']/2./blade['pf']['r'][-1] #0.025  # (Float): hub location as fraction of radius
     rotor['hub_height']       = blade['config']['hub_height']  # (Float, m): hub height
@@ -407,21 +419,24 @@ def Init_RotorGeometry_wRefBlade(rotor, blade):
 
     return rotor
 
+
 if __name__ == "__main__":
 
     # Turbine Ontology input
-    fname_input = "turbine_inputs/nrel5mw_mod_update.yaml"
+    # fname_input = "turbine_inputs/nrel5mw_mod_update.yaml"
+    fname_input = "/mnt/c/Users/egaertne/WISDEM2/wisdem/Design_Opt/nrel15mw/inputs/NREL15MW_prelim_v3.0.yaml"
 
     # Initialize blade design
     refBlade = ReferenceBlade()
-    refBlade.verbose = True
-    refBlade.NINPUT  = NINPUT
-    refBlade.NPITS   = 50
+    refBlade.verbose  = True
+    refBlade.NINPUT   = NINPUT
+    refBlade.NPTS     = 50
     refBlade.validate = False
     refBlade.fname_schema = "turbine_inputs/IEAontology_schema.yaml"
 
     refBlade.spar_var = ['Spar_Cap_SS', 'Spar_Cap_PS']
     refBlade.te_var   = 'TE_reinforcement'
+    # refBlade.le_var       = 'le_reinf'
     blade = refBlade.initialize(fname_input)
 
     # setup
